X-Git-Url: https://git.saurik.com/bison.git/blobdiff_plain/ec5479ce351b0365549e9c7d570b0ee86a4a6589..0049ec8628e5a320d055a0cea8676a58cec60986:/src/symtab.h
diff --git a/src/symtab.h b/src/symtab.h
index 1a3ada4c..7a25b360 100644
--- a/src/symtab.h
+++ b/src/symtab.h
@@ -1,6 +1,6 @@
/* Definitions for symtab.c and callers, part of Bison.
- Copyright (C) 1984, 1989, 1992, 2000, 2001, 2002, 2004, 2005, 2006
+ Copyright (C) 1984, 1989, 1992, 2000, 2001, 2002, 2004, 2005, 2006, 2007
Free Software Foundation, Inc.
This file is part of Bison, the GNU Compiler Compiler.
@@ -30,6 +30,7 @@
# include "assoc.h"
# include "location.h"
+# include "scan-code.h"
# include "uniqstr.h"
/*----------.
@@ -69,28 +70,16 @@ struct symbol
/** Any \c \%destructor declared specifically for this symbol.
Access this field only through symbol's interface functions. For
- example, if symbol::destructor = NULL, the default
- \c \%destructor or a per-type \c \%destructor might be appropriate, and
+ example, if symbol::destructor = NULL, a default \c \%destructor
+ or a per-type \c \%destructor might be appropriate, and
\c symbol_destructor_get will compute the correct one. */
- const char *destructor;
-
- /** The location of \c symbol::destructor.
-
- Access this field only through symbol's interface functions.
- \sa symbol::destructor */
- location destructor_location;
+ code_props destructor;
/** Any \c \%printer declared specifically for this symbol.
Access this field only through symbol's interface functions.
\sa symbol::destructor */
- const char *printer;
-
- /** The location of \c symbol::printer.
-
- Access this field only through symbol's interface functions.
- \sa symbol::destructor */
- location printer_location;
+ code_props printer;
symbol_number number;
location prec_location;
@@ -106,7 +95,6 @@ struct symbol
bool declared;
};
-
/** Undefined user number. */
#define USER_NUMBER_UNDEFINED -1
@@ -133,6 +121,9 @@ symbol *symbol_get (const char *key, location loc);
Its name cannot conflict with the user's names. */
symbol *dummy_symbol_get (location loc);
+/** Is this a dummy nonterminal? */
+bool symbol_is_dummy (const symbol *sym);
+
/** Declare the new symbol \c sym. Make it an alias of \c symval. */
void symbol_make_alias (symbol *sym, symbol *symval, location loc);
@@ -142,26 +133,18 @@ void symbol_make_alias (symbol *sym, symbol *symval, location loc);
void symbol_type_set (symbol *sym, uniqstr type_name, location loc);
/** Set the \c destructor associated with \c sym. */
-void symbol_destructor_set (symbol *sym, const char *destructor, location loc);
-
-/** Get the computed \c \%destructor for \c sym, or \c NULL if none. */
-const char *symbol_destructor_get (symbol *sym);
+void symbol_destructor_set (symbol *sym, code_props const *destructor);
-/** Get the grammar location of the computed \c \%destructor for \c sym.
-
- \pre symbol_destructor_get (sym) != NULL */
-location symbol_destructor_location_get (symbol *sym);
+/** Get the computed \c \%destructor for \c sym, which was initialized with
+ \c code_props_none_init if there's no \c \%destructor. */
+code_props const *symbol_destructor_get (symbol const *sym);
/** Set the \c printer associated with \c sym. */
-void symbol_printer_set (symbol *sym, const char *printer, location loc);
+void symbol_printer_set (symbol *sym, code_props const *printer);
-/** Get the computed \c \%printer for \c sym, or \c NULL if none. */
-const char *symbol_printer_get (symbol *sym);
-
-/** Get the grammar location of the computed \c \%printer for \c sym.
-
- \pre symbol_printer_get (sym) != NULL */
-location symbol_printer_location_get (symbol *sym);
+/** Get the computed \c \%printer for \c sym, which was initialized with
+ \c code_props_none_init if there's no \c \%printer. */
+code_props const *symbol_printer_get (symbol const *sym);
/* Set the \c precedence associated with \c sym.
@@ -194,15 +177,46 @@ extern symbol *startsymbol;
extern location startsymbol_location;
-/*---------------.
-| Symbol table. |
-`---------------*/
+/*-----------------.
+| Semantic types. |
+`-----------------*/
+
+/** A semantic type and its associated \c \%destructor and \c \%printer.
+
+ Access the fields of this struct only through the interface functions in
+ this file. \sa symbol::destructor */
+typedef struct {
+ /** The key, name of the semantic type. */
+ uniqstr tag;
+
+ /** Any \c %destructor declared for this semantic type. */
+ code_props destructor;
+ /** Any \c %printer declared for this semantic type. */
+ code_props printer;
+} semantic_type;
+
+/** Fetch (or create) the semantic type associated to KEY. */
+semantic_type *semantic_type_from_uniqstr (const uniqstr key);
+
+/** Fetch (or create) the semantic type associated to KEY. */
+semantic_type *semantic_type_get (const char *key);
+
+/** Set the \c destructor associated with \c type. */
+void semantic_type_destructor_set (semantic_type *type,
+ code_props const *destructor);
+
+/** Set the \c printer associated with \c type. */
+void semantic_type_printer_set (semantic_type *type,
+ code_props const *printer);
+/*----------------------------------.
+| Symbol and semantic type tables. |
+`----------------------------------*/
-/** Create the symbol table. */
+/** Create the symbol and semantic type tables. */
void symbols_new (void);
/** Free all the memory allocated for symbols. */
void symbols_free (void);
/** Check that all the symbols are defined.
@@ -217,14 +231,18 @@ void symbols_check_defined (void);
void symbols_pack (void);
-/*-----------------------------------.
-| Default %destructor and %printer. |
-`-----------------------------------*/
+/*---------------------------------------.
+| Default %destructor's and %printer's. |
+`---------------------------------------*/
-/** Set the default \c \%destructor. */
-void default_destructor_set (const char *destructor, location loc);
+/** Set the default \c \%destructor for tagged values. */
+void default_tagged_destructor_set (code_props const *destructor);
+/** Set the default \c \%destructor for tagless values. */
+void default_tagless_destructor_set (code_props const *destructor);
-/** Set the default \c \%printer. */
-void default_printer_set (const char *printer, location loc);
+/** Set the default \c \%printer for tagged values. */
+void default_tagged_printer_set (code_props const *printer);
+/** Set the default \c \%printer for tagless values. */
+void default_tagless_printer_set (code_props const *printer);
#endif /* !SYMTAB_H_ */
@@ -194,15 +177,46 @@ extern symbol *startsymbol;
void symbols_new (void);
-/** Free all the memory allocated for symbols. */
+/** Free all the memory allocated for symbols and semantic types. */
void symbols_free (void);
/** Check that all the symbols are defined.